#define ON 1
#define OFF 0

//contains all the globabl variables and definitions

#define DDR_SPI DDRB	//PORT B is the SPI port
#define DD_MOSI DD5
#define DD_MISO DD6
#define DD_SCK DD7
#define DD_CSN DD4	//Use this pin to select the RF module as the slave(CSN)
#define CSN PORTB4//
#define DD_CE DD3//chip enable CE = 
#define CE PORTB3

unsigned char status;//contains status returned by the RF module
unsigned char error;
unsigned char new_data_ready;
unsigned char en_DPL;
unsigned char RX_payload_length;
unsigned char RX_payload_array[32];
unsigned char TX_payload_array[32];

//module_API.h
#include "init_RF.h"//Initializes interface between the RF module and MCU
#include "SPI_transmit.h"
#include "CMD_LIST.h"		//Contains definition for the command list
#include "register_map.h"	//Contains definition for register

void control_CE(unsigned char input){
	if(input==ON) PORTB |= (1<<CE);// set the CE pin high
    else if(input==OFF) PORTB &= ~(1<<CE);
}
void control_CSN(unsigned char input){
//CSN pin is used to initiate and end the communication between the SPI module of mega644 and the RF chip
	if(input==ON) 	PORTB &= ~(1<<CSN);//Start the SPI communication
	else if(input==OFF) PORTB |= (1<<CSN);//end transmission
}
void write_RF_register(unsigned char addr, unsigned char value){
	unsigned char temp8;
	control_CSN(ON);
	temp8 = W_REGISTER|addr;
	SPI_transmit(temp8);
	status = SPDR;
	SPI_transmit(value);
	control_CSN(OFF);//end transmission
}



void power_up(void){
//You have to power up first before you set the RF module in PTX or PRX mode
//You can not set the device in PWR_UP and PRX or PTX mode at once
	unsigned char temp8;
	control_CSN(ON);
	temp8 = W_REGISTER |CONFIG;
	SPI_transmit(temp8);
	status = SPDR;
	//Mask all the interrupts from the RF module
	temp8 = (1<<MASK_RX_DR)|(1<<MASK_TX_DS)|(1<<MASK_MAX_RT)|(1<<PWR_UP)|(1<<EN_CRC);
	SPI_transmit(temp8);
	control_CSN(OFF);//end transmission
}

void power_down(void){
	unsigned char temp8;
	control_CSN(ON);
	temp8 = W_REGISTER |CONFIG;
	SPI_transmit(temp8);
	status = SPDR;
	//Mask all the interrupts from the RF module
	temp8 = (1<<MASK_RX_DR)|(1<<MASK_TX_DS)|(1<<MASK_MAX_RT)|(0<<PWR_UP)|(1<<EN_CRC);
	SPI_transmit(temp8);
	control_CSN(OFF);//end transmission
}
	
//All the API function returns 0 if no error, 1 if there is error


/****************************************************************************/
/****************************************************************************/
/*******Register Write Commands**********************************************/
/*********Below functions writes to the internal register on the RF module***/
/************some are used to configure, some are used to send data*********/
/****************************************************************************/

void clear_IRQs(void){
	unsigned char  temp8;
	control_CSN(ON);
	temp8 = W_REGISTER|STATUS;
	SPI_transmit(temp8);
	status = SPDR;
	SPI_transmit(0x7E);//Reset all IRQ's
	control_CSN(OFF);//end transmission
}
unsigned char TXmode(unsigned char input){
//Conofigures the RF module in PRX or PTX mode	
	//input = 0 --> PRX mode
	//input = 1--> PTX mode
	unsigned char temp8;//temp 8 = 8 bit buffer
	
	if(input>1) return 1;
	clear_IRQs();
	flush_TX_FIFO();
	power_down();
	_delay_ms(10);
	power_up();
	control_CSN(ON);
	temp8 = W_REGISTER | CONFIG;
	SPI_transmit(temp8);
	status = SPDR;
	if(input) temp8 = (0<<PRIM_RX)|(1<<PWR_UP)|(1<<EN_CRC);//TX mode
	else temp8 = (1<<PRIM_RX)|(1<<PWR_UP)|(1<<EN_CRC);//RX mode
	SPI_transmit(temp8);
	control_CSN(OFF);//end transmission

	//Now, enable the chip again
	//_delay_ms(100);
	if(input==0) control_CE(ON);//We want CE always on for PRX mode
	else control_CE(OFF); 
	return 0;

}
unsigned char setup_addr_width(unsigned char input){
//sets up Address Width
	//input = 1~3
	//1 = 3 byte, 2 = 4 byte, 3 = 5 byte

	unsigned char temp8;//temp8 = 8 bit buffer
	if(input==0||input>3) return 1;
	control_CSN(ON);
	temp8 = W_REGISTER | SETUP_AW;

	SPI_transmit(temp8);
	status = SPDR;
	SPI_transmit(input);
	control_CSN(OFF);//end transmission
	return 0;
}

unsigned char setup_auto_retr(unsigned char delay,unsigned char count){
//Sets up the Auto-Retransmission feature in case of missing NOACK
	//delay = (0 ~ 15) 0 ==> 250uS delay, 15 ==> 4ms delay
	// count = (0~15)  0 ==> no retransmission, 15==> IRQ generated after failing 15 retransmissions
	unsigned char temp8;
	if(delay>15||count>15) return 1;
	control_CSN(ON);
	temp8 = W_REGISTER | SETUP_RETR;
	SPI_transmit(temp8);
	status = SPDR;
	temp8 = (delay<<4) + count;
	SPI_transmit(temp8);
	control_CSN(OFF);//end transmission
	return 0;
}

unsigned char setup_RF_frequency(unsigned char input){
//Sets up the base frequency for RF signal
//Fo = (2400 + RF_CH)[Mhz]
//Possible to set between 2.400GHz to 2.525 GHz
	unsigned char temp8;
	if(input>125) return 1;
	control_CSN(ON);
	temp8 = W_REGISTER | RF_CH;
	SPI_transmit(temp8);
	status = SPDR;
	SPI_transmit(input);
	control_CSN(OFF);//end transmission
	return 0;
}

unsigned char setup_RF_data_rate(unsigned input){
//Sets up data rate
//input = 0~2; 0 -> 250kbps, 1->1Mbps, 2 ->2Mbps
	unsigned char temp8;
	if(input>2) return 1;
	control_CSN(ON);
	temp8 = W_REGISTER | RF_SETUP;
	SPI_transmit(temp8);
	status = SPDR;
	if(input==0) temp8 = (1<<RF_DR_LOW)|0;//6 is for meximum power
	else if(input==1) temp8 = 0;
	else temp8 = (1<<RF_DR_HIGH)|0;
	SPI_transmit(temp8);
	control_CSN(OFF);//end transmission
	return 0;
}
void read_status(void){
	//returns the value of the status register in the variable "status"
	control_CSN(ON);
	SPI_transmit(NOP);
	status = SPDR;
	control_CSN(OFF);//end transmission
}
void enable_all_pipes(void){
	unsigned char temp8;
    control_CSN(ON);
	temp8 = W_REGISTER | EN_RXADDR;
	SPI_transmit(temp8);
	status = SPDR;
	SPI_transmit(0x3F);
	control_CSN(OFF);
}
void enable_pipe(unsigned char input){
	//input = 0~5. input= 0 means enable data pipe 0
	unsigned char temp8;
	if(input>5) return;
	control_CSN(ON);
	temp8 = W_REGISTER | EN_RXADDR;
	SPI_transmit(temp8);
	status = SPDR;
	switch(input){
		case 5: SPI_transmit(0x20); break;
		case 4: SPI_transmit(0x10); break;
		case 3: SPI_transmit(0x08); break;
		case 2: SPI_transmit(0x04); break;
		case 1: SPI_transmit(0x02); break;
		case 0: SPI_transmit(0x01); break;
	}//end switch(input)
	control_CSN(OFF);
}
	
void set_RX_addr_P0(unsigned char MSByte, unsigned long LSBytes){
//for PTX devices, it has to be set the same as TX pipe address
//sets up 5 byte(maximum) RX address
//Only lower bytes will be used if user sets Address width that is lower than 5 bytes
	unsigned char temp8;
	control_CSN(ON);
	temp8 = W_REGISTER | RX_ADDR_P0;
	SPI_transmit(temp8);
	status = SPDR;
	temp8 = LSBytes;
	SPI_transmit(temp8);
	temp8 = LSBytes>>8;
	SPI_transmit(temp8);
	temp8 = LSBytes>>16;
	SPI_transmit(temp8);
	temp8 = LSBytes>>24;
	SPI_transmit(temp8);
	SPI_transmit(MSByte);
	control_CSN(OFF);//end transmission
}
void set_RX_addr_P1(unsigned char MSByte, unsigned long LSBytes){
//sets up 5 byte(maximum) RX address
//Only lower bytes will be used if user sets Address width that is lower than 5 bytes
	unsigned char temp8;
	control_CSN(ON);
	temp8 = W_REGISTER | RX_ADDR_P1;
	SPI_transmit(temp8);
	status = SPDR;
	temp8 = LSBytes;
	SPI_transmit(temp8);
	temp8 = LSBytes>>8;
	SPI_transmit(temp8);
	temp8 = LSBytes>>16;
	SPI_transmit(temp8);
	temp8 = LSBytes>>24;
	SPI_transmit(temp8);
	SPI_transmit(MSByte);
	control_CSN(OFF);//end transmission
}
void set_RX_addr_P2(unsigned char LSByte){
//sets up 5 byte(maximum) RX address
//Upper 4 byte address will be the same is that of pipe 1
//Only lower bytes will be used if user sets Address width that is lower than 5 bytes
	unsigned char temp8;
	control_CSN(ON);
	temp8 = W_REGISTER | RX_ADDR_P2;
	SPI_transmit(temp8);
	status = SPDR;
	temp8 = LSByte;
	SPI_transmit(temp8);
	control_CSN(OFF);//end transmission
}
void set_RX_addr_P3(unsigned char LSByte){
//sets up 5 byte(maximum) RX address
//Upper 4 byte address will be the same is that of pipe 1
//Only lower bytes will be used if user sets Address width that is lower than 5 bytes
	unsigned char temp8;
	control_CSN(ON);
	temp8 = W_REGISTER | RX_ADDR_P2;
	SPI_transmit(temp8);
	status = SPDR;
	temp8 = LSByte;
	SPI_transmit(temp8);
	control_CSN(OFF);//end transmission
}
void set_RX_addr_P4(unsigned char LSByte){
//sets up 5 byte(maximum) RX address
//Upper 4 byte address will be the same is that of pipe 1
//Only lower bytes will be used if user sets Address width that is lower than 5 bytes
	unsigned char temp8;
	control_CSN(ON);
	temp8 = W_REGISTER | RX_ADDR_P2;
	SPI_transmit(temp8);
	status = SPDR;
	temp8 = LSByte;
	SPI_transmit(temp8);
	control_CSN(OFF);//end transmission
}
void set_RX_addr_P5(unsigned char LSByte){
//sets up 5 byte(maximum) RX address
//Upper 4 byte address will be the same is that of pipe 1
//Only lower bytes will be used if user sets Address width that is lower than 5 bytes
	unsigned char temp8;
	control_CSN(ON);
	temp8 = W_REGISTER | RX_ADDR_P2;
	SPI_transmit(temp8);
	status = SPDR;
	temp8 = LSByte;
	SPI_transmit(temp8);
	control_CSN(OFF);//end transmission
}



void set_TX_addr(unsigned char MSByte, unsigned long LSBytes){
	//sets up 5 byte(maximum) TX pipe address
	//Only lower bytes will be used if user sets Address width that is lower than 5 bytes
	//For one to one transmission, TX address should be equal to RX address
	unsigned char temp8;
	control_CSN(ON);
	temp8 = W_REGISTER | TX_ADDR;
	SPI_transmit(temp8);
	status = SPDR;
	temp8 = LSBytes;
	SPI_transmit(temp8);
	temp8 = LSBytes>>8;
	SPI_transmit(temp8);
	temp8 = LSBytes>>16;
	SPI_transmit(temp8);
	temp8 = LSBytes>>24;
	SPI_transmit(temp8);
	SPI_transmit(MSByte);
	control_CSN(OFF);//end transmission
	control_CE(ON);//Start putting the data down the pipe
}

unsigned char set_payload_legnth(unsigned char input){
//Sets the the number of Bytes in RX payload
//input = 0~32; 0==> datapipe not used, 1~32 ==>payload length
//for now, we only use datapipe 0
	unsigned char temp8;
	if(input>32) return 1;
	RX_payload_length = input;
	control_CSN(ON);
	temp8 = W_REGISTER | RX_PW_P0;
	SPI_transmit(temp8);
	status = SPDR;
	SPI_transmit(input);
	control_CSN(OFF);//end transmission
	return 0;
}
/****************************************************************************/
/****************************************************************************/

//Read Register function
//Reads the registe value from the RF module
//Returned data are placed in read_return array



unsigned char read_RF_register(unsigned char addr){
	//Input = any register address in the RF chip
	//The function will not read 
	//The upper byte of the return value will be set to 0xFF if invalid data access is requested
	unsigned char temp8;
	if(addr>0x17&&addr<0x1C||addr>0x1D) return 0xFFFF;
	control_CSN(ON);
	temp8 = R_REGISTER|addr;
	SPI_transmit(temp8);
	status = SPDR;
	SPI_transmit(NULL);
	control_CSN(OFF);//end transmission
	temp8 = SPDR;
	return temp8;


}



unsigned char read_RX_payload(void){
//reads RX payload and saves in RX_payload array
//This function is to be executed when the RX receive IRQ is arised	
	unsigned char i;
	if(!RX_payload_length) return 1;
	control_CSN(ON);
	SPI_transmit(R_RX_PAYLOAD);
	status = SPDR;
	for(i =0;i<RX_payload_length;i++){
		SPI_transmit(NULL);	//Send null data
		RX_payload_array[i] = SPDR;
	}
	control_CSN(OFF);//end transmission
	return 0;
}

void enable_TX_NOACK(void){
//Enables TX_PAYLOAD_NOACK command
	unsigned char temp8;
	control_CSN(ON);
	temp8 = W_REGISTER|FEATURE;
	SPI_transmit(temp8);
	status = SPDR;
	SPI_transmit(0x01);
	control_CSN(OFF);//end transmission
}


unsigned char write_TX_payload(no_ack){
//writes data from TX_payload array to TX-FIFO and starts transmitting
//no_ack = 0 ==> the module do not wait for ACK signal
//no_ack = 1 ==> the module waits for ACK signal and generates
//TX_DS or MAX_RT interrupt
	unsigned char i;
	if(!RX_payload_length) return 1;
	if(no_ack>1) return 1;
	control_CSN(ON);
	if(no_ack) SPI_transmit(W_TX_PAYLOAD_NOACK);
	else SPI_transmit(W_TX_PAYLOAD);
	status = SPDR;
	for(i = 0; i<RX_payload_length;i++){
		SPI_transmit(TX_payload_array[i]);
	}
	control_CSN(OFF);//end transmission
	control_CE(ON); //start transmission(packets on the air)
	return 0;
}


	

void flush_TX_FIFO(void){
//Flushes TX_FIFO
	control_CSN(ON);
	SPI_transmit(FLUSH_TX);
	status = SPDR;
	control_CSN(OFF);//end transmission
}

void flush_RX_FIFO(void){
//Flushes RX_FIFO
	control_CSN(ON);
	SPI_transmit(FLUSH_RX);
	status = SPDR;
	control_CSN(OFF);//end transmission

}

//We do not use REUSE_TX_PL


unsigned char read_RX_payload_width(void){
//return RX payload width for the top
	unsigned char temp8;
	control_CSN(ON);
	SPI_transmit(R_RX_PL_WID);
	status = SPDR;

	temp8 = SPDR;
	if(temp8>32){
		temp8 = 0;
		flush_RX_FIFO();
		return 0;
	}
	control_CSN(OFF);//end transmission

	return temp8;
}

//We do not use W_ACK_PAYLOAD, W_TX_PAYLOAD_NOACK, and NOP


	
unsigned char check_RX_DR(void){
	read_status();
	if(status&(1<<RX_DR)){
		clear_IRQs();
	 	return 1;
	}
	else return 0;
}

unsigned char check_TX_DS(void){
	read_status();
	if(status&(1<<TX_DS)){
		clear_IRQs();
	 	return 1;
	}
	else return 0;
}

unsigned char check_MAX_RT(void){
	read_status();
	if(status&(1<<MAX_RT)){
		clear_IRQs();
	 	return 1;
	}
	else return 0;
}

	
	




	

